function [egys,dt_pulse]  = EgysWithPileup(specdat,varargin)
% function EgysWithPileup
% Do a Monte Carlo simulation of recorded photons energies with deadtime
%  
%  inputs:
%      specdat: a spectrum struct with fields egys, the x-ray energies (keV), and specnum, the number spectrum at these energies
%      mean_cps: (optional = 1e6) counts per second
%      t_integrate:  (optional = 1e-3) (seconds) the time to process pulses
%      deadtime: (optional = 1e-9) (seconds) the deadtime
%      npulses2gen: (optional = 1000); number of input pulses to generate
%  outputs: 
%      egys: (vector size(npulses2gen,1)) the total energy of recorded pulses  for each trial
%           -1 after number of recorded pulses
%     dt_pulse: the random inter-pulse times for all the incident photons
%  REA 2011-Jul-13 17:38 - 2011-Jul-24 16:04
% COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

% prep set of random pulses: random in time
nreqargs = 2;
assert(nargin>=nreqargs);

mean_cps = 1e6; % counts per second
     % compute pulses' start times 
t_integrate = 1e-3; % second
deadtime = 1e-9;
npulses2gen = 1000;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'mean_cps';         mean_cps=varargin{i+1};   assert(isscalar(mean_cps));  i=i+1;
     case 't_integrate';         t_integrate=varargin{i+1};   assert(isscalar(t_integrate));  i=i+1;
     case 'deadtime';         deadtime=varargin{i+1};   assert(isscalar(deadtime));  i=i+1;
     case 'npulses2gen';         npulses2gen=varargin{i+1};   assert(isscalar(npulses2gen));  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end
eta = deadtime*mean_cps;
spinv =  SpectrumCumulativeInverse(specdat);  % prep the energy sample calc            
dt_pulse = -(1/mean_cps)*log(rand(npulses2gen,1)); % random interpulse times
photon_energies = interp1(spinv.ys, spinv.specuminv, rand(1,npulses2gen) );
egys = EgysWithDeadtimeLoc(dt_pulse, photon_energies, deadtime, t_integrate);
egys = egys(egys > -0.1) ; % get rid of unused entries, which are marked as -1. Allow for egy=0 as a legit entry




function [egys,Q] = EgysWithDeadtimeLoc(dt_pulse, photon_energies, deadtime, t_integrate)
%   output data: egys = the energies of recorded pulses
     Q = 0; % total energy of all recorded pulses
     egys = -1*ones(numel(dt_pulse),1); % pre-allocate for speed. Calling function gets rid of negative values
     kpulse = 1; % counter into dt_pulses and photon_energies vectors
     tproc = dt_pulse(1); % tproc will increment with each pulse
     kegy = 1; % counter into the egys vector
     if abs(deadtime) < eps % make sure can increment counter
         deadtime = eps; % even with zero deadtime 
     end
     while (tproc < t_integrate) && (kegy<numel(egys))
         tend = tproc + deadtime; % initial pulse time + deadtime
         egy = 0;
             % loop for counts within the deadtime from initial time
         while (kpulse<= (numel(dt_pulse)-1) ) && (tproc < tend)
            egy = egy + photon_energies(kpulse);
            Q = Q + photon_energies(kpulse);
            kpulse = kpulse + 1;
            tproc = tproc + dt_pulse(kpulse); 
         end
         if kpulse >= (numel(dt_pulse)-1)   % make sure do not run off array
             return;
         end
         egys(kegy) = egy;
         kegy = kegy+1;
     end

