function [nkq_out,n_pulses_ave, eta]  = NKQPileupStats(specdat,Ethresholds,varargin)
% function PileupStats
% Do a Monte Carlo simulation of nbin pileup statistics with deadtime
%  
%  inputs:
%      specdat: a spectrum struct with fields egys, the x-ray energies (keV), and specnum, the number spectrum at these energies
%      Ethresholds: the threshold energies (keV)--size(nbins-1,1)
%      mean_cps: (optional = 1e6) counts per second
%      t_integrate:  (optional = 1e-3) (seconds) the time to process pulses
%      deadtime: (optional = 1e-9) (seconds) the deadtime
%      ntrials: (optional = 100);
%  outputs: 
%      nkq_out: (vector size(ntrials,nbins+1)) the number of recorded  pulses and their total energy for each trial
%       
%  REA 2011-Jul-13 17:38 - 2011-Aug-4 10:13
% COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

% prep set of random pulses: random in time
nreqargs = 2;
assert(nargin>=nreqargs);

mean_cps = 1e6; % counts per second
     % compute pulses' start times 
t_integrate = 1e-3; % second
deadtime = 1e-9;
ntrials = 100;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'mean_cps';         mean_cps=varargin{i+1};   assert(isscalar(mean_cps));  i=i+1;
     case 't_integrate';         t_integrate=varargin{i+1};   assert(isscalar(t_integrate));  i=i+1;
     case 'deadtime';         deadtime=varargin{i+1};   assert(isscalar(deadtime));  i=i+1;
     case 'ntrials';         ntrials=varargin{i+1};   assert(isscalar(ntrials));  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end
eta = deadtime*mean_cps;
n_pulses_ave = round(mean_cps*t_integrate);
dev_npulses = sqrt(n_pulses_ave);
npulses2gen = n_pulses_ave + round(5*dev_npulses); % try to make sure have enough pulses
spinv =  SpectrumCumulativeInverse(specdat);  % prep the energy sample calc
Ethresholds = [Ethresholds(:); inf];
nbins = numel(Ethresholds); % see prev line--augmented Ethresholds to simplify NKQwithDeadtime
npulses_nbins = zeros(ntrials,nbins);
Qs = zeros(ntrials,1);
for ktrial = 1:ntrials
    dt_pulse = -(1/mean_cps)*log(rand(npulses2gen,1)); % random interpulse times
    photon_energies = interp1(spinv.ys, spinv.specuminv, rand(1,npulses2gen) );
    [npulses_vec,Q]  = NKQwithDeadtime(dt_pulse, photon_energies, deadtime, t_integrate,Ethresholds);
    
        % if you do not use the mex file, comment out the line above and uncomment the line below
%     [npulses_vec,Q]  = NKQwithDeadtimeLoc(dt_pulse, photon_energies, deadtime, t_integrate,Ethresholds);
    npulses_nbins(ktrial,:) = npulses_vec(:)';
    Qs(ktrial) = Q;
       % mex function  NKQwithDeadtime.c
          % see below for matlab code and outputs
end
nkq_out = [npulses_nbins Qs];


        function [npulses_vec,Q] = NKQwithDeadtimeLoc(dt_pulse, photon_energies, deadtime, t_integrate,Ethresholds)
        %   output data: 
%            npulses_vec: a vector with the number of recorded pulses in each bin
%            Q: the total energy of input pulses during t_integrate
             nbins = numel(Ethresholds); % recall Ethresholds = [Ethresholds(:); inf];
             npulses_vec = zeros(nbins,1);
             if abs(deadtime) < eps % make sure can increment counter
                 deadtime = eps;
             end
             kpulse = 1; % counter into dt_pulses and photon_energies vectors
             tproc = dt_pulse(1);
             Q = 0;
             while tproc < t_integrate
                 tend = tproc + deadtime;
                 egy = 0;
                 while (kpulse<= numel(dt_pulse) ) && (tproc < tend)
                    egy = egy + photon_energies(kpulse);
                    Q = Q + photon_energies(kpulse);
                    kpulse = kpulse + 1;
                    tproc = tproc + dt_pulse(kpulse); 
                 end
                 assert(kpulse ~= numel(dt_pulse));   % make sure did not run off array
                 for k = 1:nbins
                    if egy < Ethresholds(k)
                       npulses_vec(k) = npulses_vec(k) + 1;
                       break;
                    end
                 end
             end
        

