/*=================================================================
 * NKQwithDeadtime.c
 * 
 * This .mex file counts the number of pulses in N energy bins with pulse pileup
 *
 * Called by NKQPileupStats.m -- see NKQwithDeadtime subfunction
 Matlab call syntax
    function [npulses_vec,Q] = NKQwithDeadtime(dt_pulse, photon_energies, deadtime, t_integrate,Ethresholds)
    inputs: 
        dt_pulse: the input inter-pulse intervals (column vector)
        photon_energies: the energies of each pulse in dt_pulse
        deadtime: (double) the deadtime
        t_integrate: the integration time
        Ethreshold: the thresholds for N bin PHA -- last value is Inf
    outputs:
          npulses_vec: a vector with the number of recorded pulses in each bin
          Q: the total energy of input pulses during t_integrate
         error if insufficient number of points in dt_sample to get t_integrate total time

REA 2011-Jul-18 11:07 - 2011-Aug-4 11:26
 COPYRIGHT:   Aprend Technology, 2011. All rights reserved.
 */
#include <math.h>
#include <float.h>
#include "mex.h"



void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
     
{

	/* INPUT PARAMETERS -- see above for definitions*/
        double *dt_pulse = 0;			/* the input inter-pulse intervals (double vector) */
        double *photon_energies = 0;			/* the input inter-pulse intervals (double vector) */
	double *deadtime_pr = 0, deadtime = 0.0;	
	double *t_integrate_pr = 0, t_integrate = 0.0;	
	double *Ethresholds = 0;	
	
	/* OUTPUT PARAMETERS */
	double *npulses_vec=0 , *Q_pr = 0, Q = 0.0;		/* output data */
	
	
	/* Other variables */
	int npulses_1=0, npulses_2=0, kpulse = 0, npulses2gen = 0, kbin = 0, nbins = 0;
	double tproc = 0.0, tend = 0.0, egy = 0.0;
	
    
    /* Check for proper number of arguments */
    
    if (nrhs != 5) { 
		mexErrMsgTxt("N2QwithDeadtime: 5 input arguments required."); 
    } else if (nlhs != 2) {
		mexErrMsgTxt("N2QwithDeadtime: 2 output arguments required."); 
    } 
	
    dt_pulse = mxGetPr(prhs[0]);
    photon_energies = mxGetPr(prhs[1]);
    deadtime_pr = mxGetPr(prhs[2]);
    deadtime = deadtime_pr[0];
    if(deadtime < FLT_MIN){
       deadtime = 1e-15;
    }
    
    t_integrate_pr = mxGetPr(prhs[3]);
    t_integrate = t_integrate_pr[0];

    Ethresholds = mxGetPr(prhs[4]);
    nbins = mxGetM(prhs[4])*mxGetN(prhs[4]); // variable to test for overrun
    
    /* Create  matrices for the return arguments */
    plhs[0] = mxCreateDoubleMatrix(1, nbins, mxREAL);
    npulses_vec = mxGetPr(plhs[0]);  /* Get pointer to the data array */
    for(kbin =0; kbin<nbins; kbin++){ // zero output data
       npulses_vec[kbin] = 0.0;
    }

    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    Q_pr = mxGetPr(plhs[1]);  /* Get pointer to the data array */
    Q = 0.0;
    
    // get size of input data so can test if try to overrun
    npulses2gen = mxGetM(prhs[0])*mxGetN(prhs[0]) - 1; // variable to test for overrun
	
    // do the processing
    tproc = dt_pulse[0]; // the running time as the pulses are processed
    while(tproc < t_integrate){
       tend = tproc + deadtime; // the end of the deadtime from the current pulse 
       egy = 0.0; // the energy for this pulse
       while( (kpulse<npulses2gen) & (tproc<tend) ){
         egy += photon_energies[kpulse];
            // Q is the total energy for all pulses including those in deadtimes
         Q += photon_energies[kpulse]; 
          kpulse = kpulse + 1;
          tproc = tproc + dt_pulse[kpulse];
       }
       if(kpulse>=npulses2gen){
          mexErrMsgTxt("N2QwithDeadtime: overran input dt_pulse array while processing");
       }
       // update counts for bin corresponding to photon energy
        for(kbin =0; kbin<nbins; kbin++){ 
          if( egy < Ethresholds[kbin]){
             ++npulses_vec[kbin]; 
             break;
          }
        }
    }	
    Q_pr[0] = Q;
    return;
}


