% P19Deadtime4.m Pulse pileup and PHA
% COPYRIGHT:  Aprend Technology, 2011. All rights reserved.
%
%% set path
workdir = 'E:\Projects\CountDetPhysics\PileupSim\';
addpath(workdir);

%% Create the spectrum
tube_voltage = 80; % keV  
[specnum,xray_energies,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');

%% set the threshold energy(s) for the PHA
specdat.Ethreshold = 55; % 2 bins          [42,60]; % three bins
nbins = numel(specdat.Ethreshold) + 1;

%% initialize the parameters for the simulation vs. deadtime
ntrials_pileup_stats = 100;
nMCtrials = 10;
ndeadtimes = 10;
mean_cps = 1e6; % seconds
t_ave = 1/mean_cps;% sec
deadtimes = linspace(0,0.5*t_ave,ndeadtimes)'; % note transpose for column vector
t_integrate = 1.5e-3; % sec

%% run the simulation
statsbar = [];
for kdead = 1:ndeadtimes
    deadtime = deadtimes(kdead);
    stats_all = []; % for simulation  data
    for ktrial = 1:nMCtrials
        nkqs= NKQPileupStats(specdat,specdat.Ethreshold,'mean_cps',mean_cps,'deadtime',deadtime, ...
              'ntrials',ntrials_pileup_stats, 't_integrate',t_integrate);
        
        stats_all(ktrial,:) = [mean(nkqs) Cov2Vector(nkqs)];
           % variables for 2 bins
           % 1.m1bar, 2.m2bar, 3.Qbar, 4.var-m1, 5.cov-m1m2, 6.var-m2, 7. cov-m1Q 8. cov-m2Q 9. var-Q
    end
    statsbar(kdead,:) = mean(stats_all);
   fprintf('.') % progress bar
   if mod(ktrial,80)==0
       fprintf('\n');
   end
end
fprintf('\n');

%% compute  N1 and N2, var(N1) var(N2) vs. deadtime
     % the parameters for the theoretical formulas
nbar_incident = mean_cps*t_integrate;
   % get bin fractions etc for spectra with pileup
etas = mean_cps*deadtimes;
fracs = zeros(ndeadtimes,nbins);
for kdead = 1:ndeadtimes
    deadtime = deadtimes(kdead);
    spout = SpectrumWithPileup(specdat,etas(kdead));
    frac = BinFracs(spout,specdat.Ethreshold);
    fracs(kdead,:) = frac(:)';
end
     % the theoretical results and the Monte Carlo simulation statistics
          % first the mean values of counts in each bin
nbars_MC = statsbar(:,1:nbins); % mean counts in each bin for all deadtimes from the Monte Carlo data
nrecord_bar = nbar_incident./(1+etas); % theoretical mean total recorded counts M
     % nbars_calc is the theoretical mean counts for each bin for all deadtimes
nbars_calc = repmat(nbar_incident./(1 + etas),1,nbins).*fracs(:,1:nbins);
     % then the variance
Nvars_MC = statsbar(:,nbins + 1 +[1 3]); % variance of each bin for all deadtimes from the Monte Carlo data
nrecord_var = nbar_incident./(1+etas).^3; % theoretical var of total recorded counts var(M)
           % theoretical variance of each bin for all deadtimes 
Nvars_calc = repmat(nrecord_bar,1,2).*fracs(:,1:nbins) + ... 
    repmat((nrecord_var - nrecord_bar),1,nbins).*(fracs(:,1:nbins).^2);
    
%% plot the data
fh = figure;
hold on
    % results are normalized by dividing by the theoretical mean counts for each bin
plot(etas,nbars_MC(:,1)/nbar_incident,'ok',etas,nbars_calc(:,1)/nbar_incident,'-k','linewidth',1.5);
plot(etas,nbars_MC(:,2)/nbar_incident,'+r',etas,nbars_calc(:,2)/nbar_incident,':r','linewidth',1.5);
plot(etas,Nvars_MC(:,1)/nbar_incident,'ob',etas,Nvars_calc(:,1)/nbar_incident,'-b','linewidth',1.5);
plot(etas,Nvars_MC(:,2)/nbar_incident,'+g',etas,Nvars_calc(:,2)/nbar_incident,':g','linewidth',1.5);
hold off
hl = legend('N1 Monte Carlo','N1 theory','N2 Monte Carlo','N2 theory', ...
    'var N1 Monte Carlo','var N1 theory','var N2 Monte Carlo','var N2 theory');
axis([0 0.5 0 1])
xlabel('mean_cps*deadtime','fontsize',14,'interpreter','none')
ylabel('avg recorded counts and variance/incident counts','fontsize',14)
set(gca,'fontsize',14,'linewidth',1.5)
set(fh,'color','w','position',[250 400 600 600])

%% export the figure as png and eps
dirname = 'E:/Projects/Blog/Posts/P19Deadtime4/';
figno = sprintf('-f%d',fh);
fname = 'MeanVarWithDeadtime';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% covariance and correlation of PHA bin counts
    % the covariance from the Monte Carlo data
covN1N2_MC = statsbar(:,1 + nbins + 2);
    % the theoretical covariance
Nrecord_bar_calc = nbar_incident./(1 + etas);
Nrecord_var_calc = nbar_incident./(1 + etas).^3;
dPoiss = nrecord_var - nrecord_bar; % 0 for Poisson r.v.
covN1N2_calc = prod(fracs(:,1:nbins),2).*dPoiss;
    % the correlation
corrN1N2MC = covN1N2_MC./prod(sqrt(Nvars_MC),2);
corrN1N2calc = covN1N2_calc./prod(sqrt(Nvars_calc),2);
fh = figure;
   % plot the covariance
subplot(121) 
plot(etas,covN1N2_MC/nbar_incident,'ob',etas,covN1N2_calc/nbar_incident,'-b','linewidth',1.5);
hl = legend('Monte Carlo','theory');
axis([0 0.5 -0.1 0.02])
xlabel('mean_cps*deadtime','fontsize',14,'interpreter','none')
ylabel('cov(N1,N2)/incident counts','fontsize',14)
text(0.1,-.09,'(a)','fontsize',14);
set(gca,'fontsize',14,'linewidth',1.2)
   % plot the correlation
subplot(122) 
plot(etas,corrN1N2MC,'ob',etas,corrN1N2calc,'-b','linewidth',1.5);
axis([0 0.5 -0.5 0.1])
xlabel('mean_cps*deadtime','fontsize',14,'interpreter','none')
ylabel('corr(N1,N2)','fontsize',14)
text(0.1,-.45,'(b)','fontsize',14);
set(gca,'fontsize',14,'linewidth',1.2)
set(fh,'color','w','position',[250 400 750 525])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'CovarianceWDeadtime';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi
